#!/usr/bin/luajit

local ffi = require 'ffi'
local C = ffi.C

ffi.cdef[[
char *getenv(const char *);
]]

local cmd = arg[1] or 'none'
local iface = C.getenv('interface')
if iface  == nil then
    print('Interface environment variable not set')
    os.exit(2)
end
local interface = ffi.string(iface)

local ip
local _ip = C.getenv('ip')
if _ip ~= nil then
    ip = ffi.string(_ip)
end

if cmd == 'init' then
    os.execute('ip link set up dev ' .. interface)
    os.execute('ifconfig ' .. interface .. ' 0.0.0.0')
elseif cmd == 'config' then
    if not ip then
        print('ip environment variable not set')
        os.exit(2)
    end
    os.execute('ip addr add dev ' .. interface .. ' scope link local ' ..
        ip .. '/16 broadcast 169.254.255.255')
elseif cmd == 'deconfig' then
    if not ip then
        print('ip environment variable not set')
        os.exit(2)
    end
    os.execute('ip addr del dev ' .. interface .. ' scope link local ' ..
        ip .. '/16 broadcast 169.254.255.255')
else
    print('Unrecognized command:  ' .. cmd)
    os.exit(2)
end

os.exit(0)
