require 'ptpv2-ffi/cdata'

local ffi = require 'ffi'
local C = ffi.C

local m = {}

function m:is_known_foreign_master(idx)
    local ci = self.clock_info
    local pid = ci.foreign[idx].foreign_master_port_identity
    local hdr = ffi.cast('msg_hdr_t *', self.clock_info.ibuf)
    local spid = hdr.source_port_identity

    if pid.port_number == spid.port_number and
        C.memcmp(pid.clock_identity, spid.clock_identity,
            ffi.sizeof(pid.clock_identity)) == 0 then
        return true
    end
end

function m:add_foreign()
    local ci = self.clock_info
    local j = ci.foreign_record_best

    -- Check to see if we already known about this master
    for i = 0, ci.number_foreign_records - 1 do
        local fi = ci.foreign[i]
        if self:is_known_foreign_master(i) then
            local fj = ci.foreign[j]

            fj.foreign_master_announce_messages =
                fi.foreign_master_announce_messages + 1
            ffi.copy(fj.announce, ci.ibuf, ffi.sizeof('announce_msg_t'))

            -- Fix up byte ordering
            fj.announce.hdr.message_length =
                C.ntohs(fj.announce.hdr.message_length)
            fj.announce.hdr.correction_field =
                self:fixup_correction(fj.announce.hdr.correction_field)
            fj.announce.hdr.sequence_id =
                C.ntohs(fj.announce.hdr.sequence_id)
            fj.announce.hdr.source_port_identity.port_number =
                C.ntohs(fj.announce.hdr.source_port_identity.port_number)
            fj.announce.current_utc_offset =
                C.ntohs(fj.announce.current_utc_offset)
            fj.announce.steps_removed = C.ntohs(fj.announce.steps_removed)
            fj.announce.grandmaster.clock_quality.offset_scaled_log_variance =
                C.ntohs(fj.announce.
                    grandmaster.clock_quality.offset_scaled_log_variance)
            fj.announce.origin_timestamp.nanoseconds =
                C.ntohl(fj.announce.origin_timestamp.nanoseconds)
            fj.announce.origin_timestamp.seconds.msb =
                C.ntohs(fj.announce.origin_timestamp.seconds.msb)
            fj.announce.origin_timestamp.seconds.lsb =
                C.ntohl(fj.announce.origin_timestamp.seconds.lsb)
            return
        end
        j = ((j + 1) % ci.number_foreign_records)
    end

    -- New foreign master
    if ci.number_foreign_records < ci.max_foreign_records then
        ci.number_foreign_records = ci.number_foreign_records + 1
    end

    local fj = ci.foreign[ci.foreign_record_i]

    -- Copy the info for the new foreign master
    local msg = ffi.cast('msg_hdr_t *', ci.ibuf)
    ffi.copy(fj.foreign_master_port_identity.clock_identity,
        msg.source_port_identity.clock_identity,
        ffi.sizeof(fj.foreign_master_port_identity.clock_identity))
    fj.foreign_master_port_identity.port_number =
        msg.source_port_identity.port_number
    fj.foreign_master_announce_messages = 0

    -- Need message for BMC algorithm
    ffi.copy(fj.announce, ci.ibuf, ffi.sizeof('announce_msg_t'))

    -- Fix up byte ordering
    fj.announce.hdr.message_length = C.ntohs(fj.announce.hdr.message_length)
    fj.announce.hdr.correction_field =
        self:fixup_correction(fj.announce.hdr.correction_field)
    fj.announce.hdr.sequence_id = C.ntohs(fj.announce.hdr.sequence_id)
    fj.announce.hdr.source_port_identity.port_number =
        C.ntohs(fj.announce.hdr.source_port_identity.port_number)
    fj.announce.current_utc_offset = C.ntohs(fj.announce.current_utc_offset)
    fj.announce.steps_removed = C.ntohs(fj.announce.steps_removed)
    fj.announce.grandmaster.clock_quality.offset_scaled_log_variance =
        C.ntohs(fj.announce.grandmaster.clock_quality
            .offset_scaled_log_variance)
    fj.announce.origin_timestamp.nanoseconds =
        C.ntohl(fj.announce.origin_timestamp.nanoseconds)
    fj.announce.origin_timestamp.seconds.msb =
        C.ntohs(fj.announce.origin_timestamp.seconds.msb)
    fj.announce.origin_timestamp.seconds.lsb =
        C.ntohl(fj.announce.origin_timestamp.seconds.lsb)

    ci.foreign_record_i = ((ci.foreign_record_i + 1) % ci.max_foreign_records)
end

-- Local clock synchronizied to Ebest (Table 16 (9.3.5) of spec)
function m:s1(hdr, announce_msg)
    local ci = self.clock_info

    ci.steps_removed = announce_msg.steps_removed + 1

    ffi.copy(ci.parent_port_identity, hdr.source_port_identity,
        ffi.sizeof(ci.parent_port_identity))

    ffi.copy(ci.grandmaster, announce_msg.grandmaster,
        ffi.sizeof(ci.grandmaster))

    ci.current_utc_offset = announce_msg.current_utc_offset

    ci.current_utc_offset_valid = bit.band(hdr.flag_field[1], 0x4)
    ci.leap59 = bit.band(hdr.flag_field[1], 0x2)
    ci.leap61 = bit.band(hdr.flag_field[1], 0x1)
    ci.time_traceable = bit.band(hdr.flag_field[1], 0x10)
    ci.frequency_traceable = bit.band(hdr.flag_field[1], 0x20)
    ci.ptp_timescale = bit.band(hdr.flag_field[1], 0x8)
    ci.time_source = announce_msg.time_source
end

-- Local clock is becoming master (Table 13 (9.3.5) of spec)
function m:m1()
    local clock_info = self.clock_info

    clock_info.steps_removed = 0
    self.offset_from_master.time.seconds = 0
    self.offset_from_master.time.nanoseconds = 0

    self.mean_path_delay.time.seconds = 0
    self.mean_path_delay.time.nanoseconds = 0

    -- Stuff used to work around 8377 timestamping buggage
    clock_info.avg_sm_delay = 0
    clock_info.num_adjusted = 0

    clock_info.num_rejected_ts = 0
    clock_info.num_double_reject_ts = 0
    clock_info.num_rejected_delay_responses = 0
    clock_info.num_rejected_delay_response_ts = 0

    ffi.copy(clock_info.parent_port_identity.clock_identity,
        clock_info.clock_identity,
        ffi.sizeof(clock_info.parent_port_identity.clock_identity))
    clock_info.parent_port_identity.port_number = 0

    clock_info.parent_stats = C.DEFAULT_PARENT_STATS

    ffi.copy(clock_info.grandmaster.identity, clock_info.clock_identity,
        ffi.sizeof(clock_info.grandmaster.identity))
    clock_info.grandmaster.clock_quality.clock_accuracy =
        clock_info.clock_quality.clock_accuracy
    clock_info.grandmaster.clock_quality.clock_class =
        clock_info.clock_quality.clock_class
    clock_info.grandmaster.clock_quality.offset_scaled_log_variance =
        clock_info.clock_quality.offset_scaled_log_variance

    clock_info.grandmaster.priority1 = clock_info.priority1
    clock_info.grandmaster.priority2 = clock_info.priority2
end

function m:bmc_data_set_comparison(anc_a, anc_b)
    local ci = self.clock_info

    -- A and B are the same grandmaster clock
    if C.memcmp(anc_a.grandmaster.identity, anc_b.grandmaster.identity,
        ffi.sizeof(anc_a.grandmaster.identity)) == 0 then
        -- Part 2 of algorithm (Figure 28)
        if anc_a.steps_removed > anc_b.steps_removed + 1 then
            return 1
        elseif anc_b.steps_removed > anc_a.steps_removed + 1 then
            return -1
        end

        -- A is within 1 of B
        if C.memcmp(anc_a.hdr.source_port_identity.clock_identity,
            ci.parent_port_identity.clock_identity,
            ffi.sizeof(anc_a.hdr.source_port_identity.clock_identity)) == 0
            then
            self:log("error", "Sender = Receiver")
            return 0
        elseif anc_a.steps_removed > anc_b.steps_removed then
            return 1
        elseif anc_b.steps_removed > anc_a.steps_removed then
            return 1
        elseif C.memcmp(anc_a.hdr.source_port_identity.clock_identity,
            anc_b.hdr.source_port_identity.clock_identity,
            ffi.sizeof(anc_a.hdr.source_port_identity.clock_identity)) < 0
            then
            return -1
        end
    end

    -- Different grandmasters
    local comp
    if anc_a.grandmaster.priority1 ~= anc_b.grandmaster.priority1 then
        comp = anc_a.grandmaster.priority1 - anc_b.grandmaster.priority1
    elseif anc_a.grandmaster.clock_quality.clock_class ~=
        anc_b.grandmaster.clock_quality.clock_class then
        comp = anc_a.grandmaster.clock_quality.clock_class -
                anc_b.grandmaster.clock_quality.clock_class
    elseif anc_a.grandmaster.clock_quality.clock_accuracy ~=
        anc_b.grandmaster.clock_quality.clock_accuracy then
        comp = anc_a.grandmaster.clock_quality.clock_accuracy -
                anc_b.grandmaster.clock_quality.clock_accuracy
    elseif anc_a.grandmaster.clock_quality.offset_scaled_log_variance ~=
        anc_b.grandmaster.clock_quality.offset_scaled_log_variance then
        comp = anc_a.grandmaster.clock_quality.offset_scaled_log_variance -
                anc_b.grandmaster.clock_quality.offset_scaled_log_variance
    elseif anc_a.grandmaster.priority2 ~=
        anc_b.grandmaster.priority2 then
        comp = anc_a.grandmaster.priority2 - anc_b.grandmanster.priority2
    else
        comp = C.memcmp(anc_a.grandmaster.identity, anc_b.grandmaster.identity,
            ffi.sizeof(anc_a.grandmaster.identity))
    end

    if comp < 0 then return -1
    elseif comp > 0 then return 1
    else return 0
    end
end

function m:bmc_state_decision(ci, foreign_best)
    if ci.slave_only ~= 0 then
        self:s1(ffi.cast('msg_hdr_t *', foreign_best), foreign_best)
        if ci.mean_path_delay.time.seconds == 0 and
            ci.mean_path_delay.time.nanoseconds == 0 then
            local f = self.send[C.PTP_MSG_TYPE_DELAY_REQ]
            f(self)
        end
        return C.PTP_STATE_SLAVE
    end

    if ci.number_foreign_records == 0 and ci.state == C.PTP_STATE_LISTENING then
        return C.PTP_STATE_LISTENING
    end

    ffi.copy(ci.amsg, ci.ibuf, ffi.sizeof('announce_msg_t'))

    ci.amsg.grandmaster.priority1 = ci.priority1
    ffi.copy(ci.amsg.grandmaster.identity, ci.clock_identity,
        ffi.sizeof(ci.clock_identity))
    ci.amsg.grandmaster.clock_quality.clock_class =
        ci.clock_quality.clock_class
    ci.amsg.grandmaster.clock_quality.clock_accuracy =
        ci.clock_quality.clock_accuracy
    ci.amsg.grandmaster.clock_quality.offset_scaled_log_variance =
        ci.clock_quality.offset_scaled_log_variance
    ci.amsg.grandmaster.priority2 = ci.priority2
    ci.steps_removed = 0

    local status = self:bmc_data_set_comparison(ci.amsg,
        ffi.cast('announce_msg_t *', ci.ibuf))

    if status == 0 then
        self:log("error", "Error doing BMC data set comparison")
        return C.PTP_STATE_FAULTY
    elseif status < 0 then
        self:m1()
        return C.PTP_STATE_MASTER
    elseif ci.clock_quality.clock_class < 128 then
        self:s1(ffi.cast('msg_hdr_t *', ci.amsg), ci.amsg)
        return C.PTP_STATE_PASSIVE
    else
        self:s1(ffi.cast('msg_hdr_t *', ci.amsg), ci.amsg)
        if self.mean_path_delay.seconds == 0 and
            self.mean_path_delay.nanoseconds == 0 then
            local f = self.send[C.PTP_MSG_TYPE_DELAY_REQ]
            f(self)
        end
        return C.PTP_STATE_SLAVE
    end
end

function m:best_master_clock()
    local best = 0
    local ci = self.clock_info

    if ci.number_foreign_records == 0 and ci.state == C.PTP_STATE_MASTER then
        self:m1()
        return ci.state
    end

    for i = 1, ci.number_foreign_records - 1 do
        if self:bmc_data_set_comparison(ci.foreign[i].announce,
            ci.foreign[best].announce) < 0 then
            best = i
        end
    end

--[=[
    -- Workaround for 8377 hardware timestamping buggage.  If we are in slave
    -- mode, save off the master clock identity for use after running the best
    -- master clock algorithm.
    local old_state = ci.state
    if ci.state == C.PTP_STATE_SLAVE and ci.parent_port_identity then
        ffi.copy(ci.old_ppi, ci.parent_port_identity, ffi.sizeof(ci.old_ppi))
    end
]=]

    local new_state = self:bmc_state_decision(ci, ci.foreign + best)

--[=[
    -- Workaround for 8377 hardware timestamping buggage.  If we were in slave
    -- state before running the bmc algorithm and are still in slave state, and
    -- the master clock has changed, reset the stuff used to work around the
    -- 8377 PTP timestamping issue.
    if old_state == new_state and old_state == C.PTP_STATE_SLAVE then
        if C.memcmp(ci.old_ppi.clock_identity,
            ci.parent_port_identity.clock_identity,
            ffi.sizeof(ci.parent_port_identity.clock_identity)) ~= 0 then

            ci.avg_sm_delay = 0
            ci.num_adjusted = 0
            ci.num_sm_delay_entries = 0

            ci.num_rejected_ts = 0
            ci.num_double_reject_ts = 0
            ci.num_rejected_delay_responses = 0
            ci.num_rejected_delay_response_ts = 0
        end
    end
]=]

    return new_state
end

return m
