--[[
	Misc utilities file.
]]

local log = require "log"
log:open('mdb_translation' ,log.USER )
local sys_notify = require "sys_notify"

local utilities = {}

function utilities:failed( msg )
    sys_notify:new('SDBX Import Failed'):warning(msg)
    os.exit(1)
end

function utilities:trash_alert( count)
	sys_notify:new( 'Unrecognized line format' ):warning( count
		..' of the SDBX file lines were not loaded due to being unrecognizable.')
end

function utilities:fin_wtih_errors( msg )
	sys_notify:new('SDBX Import succeeded with complicatins.'):error(msg)
end

function utilities:incomplete( fn)
	sys_notify:new( 'SDBX file "'..fn
		..'" imported with warnings.' ):warning( 'Refer to the system log for details.' )
end

function utilities:notice( msg)
	log:notice( msg)
end

function utilities:write_files( files)
	for file_name,file_content in pairs( files) do
		os.execute( 'echo "" > "' .. file_name .. '.sxr"' )
		local fh = assert( io.open( file_name..'.sxr' , 'w' ) )
		fh:write( file_content)
		fh:close()
	end
end		

function utilities:parse_filename( text)
	local root_ext = text:match( ".+/(.+)" ) or text
	local root = text:match( ".+/(.+)%." ) or text
	return { full=text ,root=root ,root_with_ext=root_ext}
end

function utilities:read( file_name )
	local possible_file = io.open( file_name , "r" )
	local text
	if possible_file then
		text = possible_file:read("*a")
	end
	io.close()
	return text
end

function utilities:make_storage()
	local check = io.popen( 'ls /storage'):read('*all')
	io.close()

	if not check:match( 'sdbx') then
		os.execute( 'mkdir /storage/sdbx')
	end
end

function utilities:kv_size( t)
	local count = 0
	for k,v in pairs( t) do
		count = count + 1
	end
	return count
end


--=======================================================

function copy_table( original_table )
	local new_table = {}

	for i,v in pairs( original_table ) do
		if type( v ) ~= "table" then
			new_table[i] = v
		else
			new_table[i] = copy_table( original_table[i] )
		end
	end

	return new_table
end

function depth_print( target , pad ) --for debugging
	if not pad then pad = "" end
	for k,v in pairs( target ) do
		print( pad , k , v )
		if type(v) == 'table' then
			depth_print( v , pad .. "\t" )
		end
	end
end




return utilities
