local cjson = require 'cjson.safe'

local testing

-- Function used to get the platform information from a device
local function query_platform_info(network, device)
    local rrs = network:QueryLocalRR(device, 'TXT', '*', '*')
    if not rrs then return platform_info end
    for _, rr in ipairs(rrs) do
        local tag, json = rr[3]:match('(.-:) (.+)')
        if tag and json and tag == 'platform_info:' then
            return cjson.decode(json)
        end
    end
    return
end

-- Function used to get the platform information for ALL devices
local function get_platform_info(network)
    local platform_info = {}

    -- First grab a list of devices and their IP addresses
    local rrs = network:QueryLocalRR('*', 'A', '*', '*')
    if not rrs then return platform_info end
    for _, rr in ipairs(rrs) do
        platform_info[rr[1]] = query_platform_info(network, rr[1])
    end
    return platform_info
end

if testing then
    local ev = require 'ev'
    local dbus = require 'dbus'

    local bus = dbus.bus_get('system')
    bus:hook_libev_default_loop()

    local network = assert(bus:auto_proxy_wait('com.somat.armada.Network',
        '/com/somat/armada/Network'))

    local function json_format(json)
        local ret, c, d = "", "", ""
        local inquotes = false
        local tabs = 0
        local tab = function(i) return string.rep(" ", i*4) end
        local function separators(c)
            local i = tabs
            local t = {
                ['{'] = not inquotes and '{\n'..tab(i) or '{',
                ['}'] = not inquotes and '\n'..tab(i)..'}' or '}',
                [','] = (not inquotes and d ~= "{") and ',\n'..tab(i) or ',',
                [':'] = not inquotes and ': ' or ":",
                [''] = ' ',
            }
            return t[c] or c
        end
        for i=1,#json do
            c = json:sub(i,i)
            d = json:sub(i+1,i+1)
            if c == '"' then inquotes = not inquotes end
            tabs = c == "{" and tabs+1 or c == "}" and tabs-1 or tabs
            ret = ret .. separators(c)
        end
        return ret
    end


    ev.Idle.new(function(l, t, e)
            local hi = get_platform_info(network)
            print("HARDWARE INFORMATION")
            print(json_format(cjson.encode(hi)))
            t:stop(l)
            l:unloop()
        end):start(ev.Loop.default)

    ev.Loop.default:loop()
end

return get_platform_info
