local cjson = require 'cjson'
local m = {}
local titlesAllChan = { "Connector","TEDS", "SDB", "Name", "Description",
                "Collect", "Sample rate", "Measurand", "Range min",
                "Range max", "Reading", "Decimals", "Units", "Data type"
                }
local titlesAnalog = {
                        "Connector","TEDS", "SDB", "Name",
                        "Input mode", "Sample rate","Filter type", 
                        "Filter frequency", "Transducer power", 
                        "Bridge Ohms", "Leadwire Ohms", "Excitation mode",
                        "Excitation range"
                    }
local titlesDigitalIn = {
                        "Connector", "Name", "Sample rate", "Threshold mode",
                        "Low threshold(mV)", "High threshold(mV)", "Transducer power"
                        }
local titlesAll = {
                    "Connector","TEDS", "SDB", "Name", "Description",
                    "Collect", "Sample rate", "Measurand", "Range min",
                    "Range max", "Decimals", "Units", "Data type",
                    "Input mode","Filter type",
                    "Filter frequency", "Transducer power",
                    "Bridge Ohms", "Leadwire Ohms", "Excitation mode",
                    "Excitation range",
                    "Threshold mode","Low threshold(mV)", "High threshold(mV)"
                }
--[[                "Threshold mode", "Low threshold", "High Threshold",
                "Output mode", "Initial State", "Final state",
                "InputChannel", "Sensor input mode", 
                "Thermocouple type", "Expiration time", 
                "Invalid value", "Request msg", "Request rate",
                "Hardware Interface", "Bit rate", "Termination",
                "AUX Port Mode", "Frame rate", "Image Size",
                "Image compression" 
                }--]]
local titleKeysAllChan = {"connector", "teds_sensor", "sdb", "name",
                    "description", "collect", "sample_rate",
                    "measurand", "range_min", "range_max", "reading_",
                    "decimals", "output_units", "data_type"
                    } 
local titleKeysAnalog = {
                        "connector", "teds_sensor", "sdb", "name",
                        "sensor_input","sample_rate", "digital_filter_type", 
                        "digital_filter_attenuation_frequency", 
                        "transducer_power", "bridge_resistance",
                        "bridge_lead_wire_resistance", "excitation_mode",
                        "excitation_range"
                    }
local titleKeysDigitalIn = {
                            "connector", "name", "smaple_rate",
                            "theshold", "transducer_power"
                        }
local titleKeysAll = {
                        "connector", "teds_sensor", "sdb", "name",
                        "description", "collect", "sample_rate",
                        "measurand", "range_min", "range_max",
                        "decimals", "output_units", "data_type",
                        "sensor_input", "digital_filter_type", 
                        "digital_filter_attenuation_frequency", 
                        "transducer_power", "bridge_resistance",
                        "bridge_lead_wire_resistance", "excitation_mode",
                        "excitation_range",
                        "threshold", "transducer_power"
                    }
                    --[["threshold", "threshold", "threshold", 
                    "output_mode", "initial_state", "final_state", 
                    "input_channels", "sensor_input_mode_", "thermocouple_type_", 
                    "stale_data_expiration_time", "invalid_data_output_value",
                    "request_msg_", "request_rate_", "hardware_interface", 
                    "bit_rate", "internal_termination", "aux_port_mode",
                    "frame_rate_", "image_size_", "image_compression_"
                    }--]]
local function writeSummary(w, t)
    local i = 0
    local widthK = 0
    local widthV = 0
    for k,v in pairs(t) do
        w:write(i, 0, k)
        w:write(i, 1, v)
        if string.len(tostring(k)) > widthK then widthK = string.len(tostring(k)) end
        if string.len(tostring(v)) > widthV then widthV = string.len(tostring(v)) end
        i = i + 1
    end
    w:set_column(0,0,widthK)
    w:set_column(1,1,widthV)
end

local function print_xlsx(t, fileName)
    local Workbook = require "xlsxwriter.workbook"
    local workbook  = Workbook:new(fileName)
    local summaryWorksheet = workbook:add_worksheet("Summary")
    local worksheet = workbook:add_worksheet("All Channels")
--    local analogWs = workbook:add_worksheet("Analog")
--    local digitalInWs = workbook:add_worksheet("Digital Input")
    local i = 0
    local width = 0
    local widthAll = {}
    for k,v in pairs (titlesAll) do
        worksheet:write(0, i, v)
        table.insert(widthAll, string.len(tostring(v)))
        worksheet:set_column(i,i,widthAll[i+1])
        i = i + 1
    end
    for k,v in pairs (t) do
        local j = 24
        if k == 1 then
            writeSummary(summaryWorksheet, v)
        else
            for k2, v2 in pairs(v) do
                i = 0
                for k3, v3 in pairs(titleKeysAll) do
                    if v3 == k2 then
                        if type(v2) == "table" then
                            if k2 == "threshold" then
                                worksheet:write(k-1, i, v2.mode)
                                worksheet:write(k-1, i+1, tostring(v2.min)..tostring(v2.units))
                                worksheet:write(k-1, i+2, tostring(v2.max)..tostring(v2.units))
                                local width1, width2 = string.len(tostring(v2.min)..tostring(v2.units)),
                                    string.len(tostring(v2.max)..tostring(v2.units))
                                if width1 > widthAll[i+2] then
                                    widthAll[i+2] = width1
                                    worksheet:set_column(i+1,i+1,widthAll[i+2])
                                 end
                                if width2 > widthAll[i+3] then
                                    widthAll[i+3] = width
                                    worksheet:set_column(i+2,i+2,widthAll[i+3])
                                 end
                                
                            else
                            worksheet:write(k-1, i, cjson.encode(v2))
                            width = string.len(cjson.encode(v2))
                            end
                        else
                            worksheet:write(k-1, i,v2)
                            width = string.len(tostring(v2))
                        end
                        if width > widthAll[i+1] then
                            widthAll[i+1] = width
                            worksheet:set_column(i,i,widthAll[i+1])
                        end
                        v2 = nil
                    end
                    i = i + 1
                end 
                if v2 ~= nil then
                    if type(v2) == "table" then
                        worksheet:write(k-1, j, k2.." : "..cjson.encode(v2))
                        width = string.len(tostring(k2).." : "..cjson.encode(v2))
                    else
                        worksheet:write(k-1, j, k2.." : "..tostring(v2))
                        width = string.len(tostring(k2).." : "..tostring(v2))
                    end
                    if widthAll[j+1] == nil then
                        table.insert(widthAll, width)
                    else
                        if width > widthAll[j+1] then
                            widthAll[j+1] = width
                        end
                    end
                    worksheet:set_column(j,j,widthAll[j+1])
                    j= j + 1
                end --]]
            end
        end
    end
    workbook:close()
end

-- Converts a Microsoft Excel .xlsx file to a test setup specification.
--
-- Parameters:
--     filename - A temporary filename from which .xlsx data should be read.
--
-- Returns:
--     On success, returns true, setup_spec.
--     On failure, returns false, error_message.

function m.import(filename)

    local f = io.open(filename, 'rb')
    if not f then return false, 'Failed to open temp file' end

    local data = f:read('*a')
    f:close()

    -- replace second return value with converted test setup table
    return true, { data = data }

end

-- Converts a test setup specification into a Microsoft Excel .xlsx file.
--
-- Parameters:
--     setup - A Lua table containing the test setup.
--     filename - A temporary filename to which .xlsx data should be written.
--
-- Returns:
--     On success, returns true. On failure, returns false, error_message.

function m.export(setup, filename)
    print_xlsx(setup, filename)
    return true
end

return m
